from collections import namedtuple
from random import randrange


TestInfo = namedtuple('TestInfo',
                      ('course', 'exam', 'date', 'note'))

Question = namedtuple('Question',
                      ('title', 'parameters', 'main_text', 'parts'))

Part = namedtuple('Part',
                  ('text', 'choices', 'correct_idx', 'points'))


def create_usual_part(text, answer, points):

    correct_idx = randrange(5)

    mid = answer / (0.8 + correct_idx * 0.1)

    choices = (0.8 * mid, 0.9 * mid, 1.0 * mid, 1.1 * mid, 1.2 * mid)

    return Part(text, choices, correct_idx, points)



def create_usual_question(title, generate_parameters, get_main_text,
                          get_part_list):

    parameters = generate_parameters()

    parts = tuple(get_part(parameters) for get_part in get_part_list)

    main_text = get_main_text(parameters)

    return Question(title, parameters, main_text, parts)


def read_question(question_list):
    ''' maybe ineffective '''

    tmp_question = Question(*question_list)

    parts = tuple(Part(*part_list) for part_list in tmp_question.parts)

    question  = Question(tmp_question.title, tmp_question.parameters,
                         tmp_question.main_text, parts)

    return question



import csv
import json
from functools import partial


def generate_student_questions(question_generators, student_data, test_id):

    name, surname, student_id = student_data

    student = {'name': name, 'surname': surname,
               'student_id': student_id, 'test_id': test_id}

    student['questions'] = [generate_question()
                            for generate_question in question_generators]

    return student


def create_exam(student_list, test_path, testinfo, question_generators):

    test = dict()

    with open(student_list, 'rb') as csvf:

        reader = csv.reader(csvf)

        test['info'] = testinfo

        generate_questions = partial(generate_student_questions,
                                     question_generators)

        test['students'] = [generate_questions(student_data, test_id)
                            for test_id, student_data in enumerate(reader)]

    with open(test_path, 'wb') as testf:
        testf.write(json.dumps(test))



from pymcq.mcqtypes import read_question, TestInfo

import json


def create_title(test, student):
    '''
    test - json dictionary
    student - json dictionary
    '''

    testinfo = TestInfo(*test['info'])
    course, exam, date, note = testinfo

    test_id = student['test_id']

    name = student['name']
    surname = student['surname']
    student_id = student['student_id']

    exam_title = r'''
    \begin{tabular}{l c}
        \begin{minipage}{0.1\textwidth}
            \begin{center}
            \includegraphics[width=\textwidth]{img/logo.jpg}
        \end{center}
        \end{minipage}
        &
        \begin{minipage}{\textwidth}
            \textbf{VISOKA TEHNICKA SKOLA U BJELOVARU}

            \textbf{STUDIJ MEHATRONIKE}

            \Large \textbf{%s - %s, %s}
        \end{minipage}
    \end{tabular}
    \vspace{0.5cm}

    \begin{tabularx}{\textwidth}{X r}
        \large \textbf{Test %s, %s %s, %s}
        &
        \normalsize \textbf{Potpis:} -------------------------------------- \\
    \end{tabularx}
    \vspace{0.2cm}

    %s
    \vspace{0.5cm}

    ''' % (course, exam, date, test_id, name, surname, student_id, note)

    return exam_title


def write_test_questions(writeline, questions):
    writeline(r"\begin{questions}")

    for question in questions:

        writeline(r"\question " + question.main_text)

        writeline(r"\begin{parts}")

        for part in question.parts:
            writeline(r"\part " + part.text)
            writeline('')

            writeline(r"\begin{oneparchoices}")
            for idx, choice in enumerate(part.choices):
                writeline(r"\CorrectChoice " if idx == part.correct_idx
                          else r"\choice")
                writeline("$" + latex_format(choice) + "$")
            writeline(r"\end{oneparchoices}")

        writeline(r"\end{parts}")

    writeline(r"\end{questions}")


def write_matrix_choices(writeline, questions):
    writeline(r"$\phantom{x}\hspace{42pt}A\hspace{12pt}B\hspace{12pt}C\hspace{12pt}D\hspace{12pt}E$")

    writeline(r"\doublespacing")
    writeline(r"\begin{questions}")

    for question in questions:

        writeline(r"\question")
        writeline(r"\begin{parts}")
        for _ in question.parts:
            writeline(r"\part")
            writeline(r"$\phantom{x}\bigcirc\phantom{x}\bigcirc\phantom{x}\bigcirc\phantom{x}\bigcirc\phantom{x}\bigcirc$")
        writeline(r"\end{parts}")

    writeline(r"\end{questions}")
    writeline(r"\singlespacing")


def create_tex(test_json, tex_path, write_questions, answers=False):

    with open(test_json, 'rb') as jsonf, open(tex_path, 'wb') as testf:

        writeline = lambda line: testf.write(line + '\n')

        test = json.loads(jsonf.read())

        writeline(r"\documentclass[answers]{exam}"
                  if answers else
                  r"\documentclass{exam}")

        writeline(r'''
        \input{header.tex}

        \begin{document}
        ''')

        for student in test['students']:

            writeline(create_title(test, student))

            questions = map(read_question, student['questions'])
            write_questions(writeline, questions)

            writeline(r"\clearpage")

        writeline(r"\end{document}")


def create_test(test_json, tex_path):
    create_tex(test_json, tex_path, write_test_questions)


def create_answers(test_json, tex_path):
    create_tex(test_json, tex_path, write_test_questions, answers=True)


def create_matrix(test_json, tex_path):
    create_tex(test_json, tex_path, write_matrix_choices)


def latex_format(x, max_exponent=2, decimal_places=2, sign=False):

    scientific_string = "{0:e}".format(x).split('e')

    mantissa = float(scientific_string[0])
    exponent = int(scientific_string[1])

    if abs(exponent) > max_exponent:
        mantissa = round(mantissa, decimal_places)
        strx = r"%s \times 10^{%s}" % (mantissa, exponent)
    else:
        number = mantissa * 10 ** exponent

        strx = '%s' % round(number, decimal_places - exponent)

    return '+' + strx if (x >= 0 and sign) else strx



import sys
import json
import csv
from itertools import izip, imap

from pymcq.mcqtypes import read_question


def grade_question(marked_idxs, correct_idx, points):
    if len(marked_idxs) == 1:
        marked_idx = marked_idxs[0]
        return points if marked_idx == correct_idx else -0.25 * points
    else:
        return 0


def get_marked_marked_idxs(marks):
    return [idx for idx, mark in enumerate(marks) if mark]


def main():
    json_path = sys.argv[1]
    csv_path = sys.argv[2]

    with open(json_path, 'rb') as json_file, open(csv_path, 'rb') as csv_file:
        test = json.loads(json_file.read())

        studenti = test['students']

        csv_reader = csv.reader(csv_file)

# ORDER MATTERS !!!!!!!!!!!!
        for row, student in izip(csv_reader, studenti):
            marked = map(int, row[4:])

# group by 5 questions
            shallow_iters = [iter(marked)] * 5
            answers = zip(*shallow_iters)

            marked_idxs = map(get_marked_marked_idxs, answers)

            correct_idxs, points = \
                zip(*[(part.correct_idx, part.points) for question
                      in map(read_question, student['questions'])
                      for part in question.parts])

            maximum = sum(points)

            total = sum(imap(grade_question,
                             izip(marked_idxs, correct_idxs, points)))

            print "%s, %s / %s" % (student['surname'], total, maximum)


if __name__ == '__main__':
    main()
